@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 #ifndef MUPDF_FITZ_FONT_H
 #define MUPDF_FITZ_FONT_H
@@ -27,6 +27,7 @@
 #include "mupdf/fitz/context.h"
 #include "mupdf/fitz/geometry.h"
 #include "mupdf/fitz/buffer.h"
+#include "mupdf/fitz/color.h"
 
 /* forward declaration for circular dependency */
 struct fz_device;
@@ -61,6 +62,44 @@ int fz_unicode_from_glyph_name_strict(const char *name);
 const char **fz_duplicate_glyph_names_from_unicode(int unicode);
 const char *fz_glyph_name_from_unicode_sc(int unicode);
 
+/**
+ * A text decoder (to read arbitrary encodings and convert to unicode).
+ */
+typedef struct fz_text_decoder fz_text_decoder;
+
+struct fz_text_decoder {
+	// get maximum size estimate of converted text (fast)
+	int (*decode_bound)(fz_text_decoder *dec, unsigned char *input, int n);
+
+	// get exact size of converted text (slow)
+	int (*decode_size)(fz_text_decoder *dec, unsigned char *input, int n);
+
+	// convert text into output buffer
+	void (*decode)(fz_text_decoder *dec, char *output, unsigned char *input, int n);
+
+	// for internal use only; do not touch!
+	void *table1;
+	void *table2;
+};
+
+/* Initialize a text decoder using an IANA encoding name.
+ * See source/fitz/text-decoder.c for the exact list of supported encodings.
+ * Will throw an exception if the requested encoding is not available.
+ *
+ * The following is a subset of the supported encodings (see source/fitz/text-decoder.c for the full list):
+ *   iso-8859-1
+ *   iso-8859-7
+ *   koi8-r
+ *   euc-jp
+ *   shift_jis
+ *   euc-kr
+ *   euc-cn
+ *   gb18030
+ *   euc-tw
+ *   big5
+ */
+void fz_init_text_decoder(fz_context *ctx, fz_text_decoder *dec, const char *encoding);
+
 /**
 	An abstract font handle.
 */
@@ -118,6 +157,9 @@ typedef struct
 
 	unsigned int cjk : 1;
 	unsigned int cjk_lang : 2; /* CNS, GB, JAPAN, or KOREA */
+
+	unsigned int embed : 1;
+	unsigned int never_embed : 1;
 } fz_font_flags_t;
 
 /**
@@ -380,6 +422,7 @@ const unsigned char *fz_lookup_noto_music_font(fz_context *ctx, int *len);
 const unsigned char *fz_lookup_noto_symbol1_font(fz_context *ctx, int *len);
 const unsigned char *fz_lookup_noto_symbol2_font(fz_context *ctx, int *len);
 const unsigned char *fz_lookup_noto_emoji_font(fz_context *ctx, int *len);
+const unsigned char *fz_lookup_noto_boxes_font(fz_context *ctx, int *len);
 
 /**
 	Try to load a fallback font for the
@@ -414,8 +457,9 @@ fz_font *fz_load_fallback_font(fz_context *ctx, int script, int language, int se
 fz_font *fz_new_type3_font(fz_context *ctx, const char *name, fz_matrix matrix);
 
 /**
-	Create a new font from a font
-	file in memory.
+	Create a new font from a font file in memory.
+
+	Fonts created in this way, will be eligible for embedding by default.
 
 	name: Name of font (leave NULL to use name from font).
 
@@ -434,6 +478,8 @@ fz_font *fz_new_font_from_memory(fz_context *ctx, const char *name, const unsign
 /**
 	Create a new font from a font file in a fz_buffer.
 
+	Fonts created in this way, will be eligible for embedding by default.
+
 	name: Name of font (leave NULL to use name from font).
 
 	buffer: Buffer to load from.
@@ -449,6 +495,8 @@ fz_font *fz_new_font_from_buffer(fz_context *ctx, const char *name, fz_buffer *b
 /**
 	Create a new font from a font file.
 
+	Fonts created in this way, will be eligible for embedding by default.
+
 	name: Name of font (leave NULL to use name from font).
 
 	path: File path to load from.
@@ -468,6 +516,11 @@ fz_font *fz_new_base14_font(fz_context *ctx, const char *name);
 fz_font *fz_new_cjk_font(fz_context *ctx, int ordering);
 fz_font *fz_new_builtin_font(fz_context *ctx, const char *name, int is_bold, int is_italic);
 
+/**
+	Control whether a given font should be embedded or not when writing.
+*/
+void fz_set_font_embedding(fz_context *ctx, fz_font *font, int embed);
+
 /**
 	Add a reference to an existing fz_font.
 
@@ -705,8 +758,9 @@ struct fz_font
 
 	int glyph_count;
 
-	/* per glyph bounding box cache */
-	fz_rect *bbox_table;
+	/* per glyph bounding box cache. */
+	fz_rect **bbox_table;
+	int use_glyph_bbox;
 
 	/* substitute metrics */
 	int width_count;
@@ -714,7 +768,7 @@ struct fz_font
 	short *width_table; /* in 1000 units */
 
 	/* cached glyph metrics */
-	float *advance_cache;
+	float **advance_cache;
 
 	/* cached encoding lookup */
 	uint16_t *encoding_cache[256];
@@ -722,6 +776,40 @@ struct fz_font
 	/* cached md5sum for caching */
 	int has_digest;
 	unsigned char digest[16];
+
+	/* Which font to use in a collection. */
+	int subfont;
 };
 
+void fz_ft_lock(fz_context *ctx);
+
+void fz_ft_unlock(fz_context *ctx);
+
+/* Internal function. Must be called with FT_ALLOC_LOCK
+ * held. Returns 1 if this thread (context!) already holds
+ * the freeetype lock. */
+int fz_ft_lock_held(fz_context *ctx);
+
+/* Internal function: Extract a ttf from the ttc that underlies
+ * a given fz_font. Caller takes ownership of the returned
+ * buffer.
+ */
+fz_buffer *fz_extract_ttf_from_ttc(fz_context *ctx, fz_font *font);
+
+/* Internal function: Given a ttf in a buffer, create a subset
+ * ttf in a new buffer that only provides the required gids.
+ * Caller takes ownership of the returned buffer.
+ *
+ * EXPERIMENTAL AND VERY SUBJECT TO CHANGE.
+ */
+fz_buffer *fz_subset_ttf_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids, int symbolic, int cidfont);
+
+/* Internal function: Given a cff in a buffer, create a subset
+ * cff in a new buffer that only provides the required gids.
+ * Caller takes ownership of the returned buffer.
+ *
+ * EXPERIMENTAL AND VERY SUBJECT TO CHANGE.
+ */
+fz_buffer *fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids, int symbolic, int cidfont);
+
 #endif
